unit MainUnit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    DeviceCombo: TComboBox;
    Panel2: TPanel;
    GoButton: TButton;
    StopButton: TButton;
    Panel3: TPanel;
    MillisecondCombo: TComboBox;
    Label2: TLabel;
    LogMemo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  protected
    //procedure IdleQueue(Sender: TObject; var Done: Boolean);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB;

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;

type
  TDACEntry = packed record
    Channel: Word;
    Value: Word;
  end;
  TPDACEntry = ^TDACEntry;

  TDACMultiCallback = procedure (pDACData: TPDACEntry; DACDataCount: PLongWord); cdecl;

function HWave_Start(DeviceIndex: LongWord; Milliseconds: Double): LongWord; cdecl; external 'HWave.dll';
function HWave_Stop(DeviceIndex: LongWord): LongWord; cdecl; external 'HWave.dll';
function HWave_RegisterCallback(DeviceIndex: Longword; pCallback: TDACMultiCallback): LongWord; cdecl; external 'HWave.dll';

procedure DACallback(pDACData: TPDACEntry; DACDataCount: PLongWord); cdecl; forward;

var
  MidScale: array of TDACEntry;
  DeviceIndex: LongWord;
  BaseClock: Integer;
  //BusyCt, MostBusyCt, MostQueueCt, FirstQueueCt: Integer;

procedure TForm1.FormCreate(Sender: TObject);
const
  NoDeviceStr = '';

var
  iDI, iDAC: Integer;
  DeviceMask: LongWord;
  Status, PID, NomSize: LongWord;
  Nom: AnsiString;
  DeviceLine: UnicodeString;

begin
  LoadDefaultFont(Font);
  DeviceCombo.Align := alTop;

  //Prep array to set all DACs to mid-scale, via DACMultiDirect.
  SetLength(MidScale, 16);
  for iDAC := 0 to 15 do MidScale[iDAC].Channel := iDAC;
  for iDAC := 0 to 15 do MidScale[iDAC].Value := $8000;

  //Stuff device combo.
  DeviceMask := GetDevices;
  for iDI := 0 to 31 do begin
    if (DeviceMask and (1 shl iDI)) = 0 then begin
      //No device at this index, put a gap at this slot so ItemIndex = DeviceIndex.
      DeviceLine := NoDeviceStr;
    end
    else begin
      NomSize := 256;
      SetString(Nom, nil, NomSize);
      Status := QueryDeviceInfo(iDI, @PID, @NomSize, @Nom[1], nil, nil);
      if Status <> ERROR_SUCCESS then begin
        DeviceLine := 'Error ' + IntToStr(Status) + ' From QueryDeviceInfo';
      end
      else begin
        SetLength(Nom, NomSize);
        DeviceLine := UnicodeString(Nom) + ' [' + IntToHex(PID, 4) + ']';
      end;
    end;

    DeviceCombo.Items.Add(DeviceLine);
  end;

  //There will normally be a bunch of gaps at the end of the list, but it's only
  //useful to keep gaps in the middle. So get rid of gaps at the end.
  for iDI := DeviceCombo.Items.Count-1 downto 1 do
    if DeviceCombo.Items[iDI] = NoDeviceStr then
      DeviceCombo.Items.Delete(iDI)
    else
      Break
  ;

  //If we have any actual devices, select the first and get ready to go.
  if DeviceCombo.Items.Count > 0 then begin
    DeviceCombo.ItemIndex := 0;
    GoButton.Enabled := True;
    ActiveControl := GoButton;
  end;

  with MillisecondCombo do ItemIndex := Items.IndexOf(Text);
end;

procedure TForm1.GoButtonClick(Sender: TObject);
var
  Status: LongWord;
  Milliseconds: LongInt;

begin
  DeviceIndex := DeviceCombo.ItemIndex;
  if DeviceIndex >= $80000000 then Exit;

  Milliseconds := StrToInt(MillisecondCombo.Text);

  //Set the DACs to mid-scale in prep for sine wave, and turn them on.
  DACMultiDirect(DeviceIndex, @MidScale[0], Length(MidScale));
  DACSetBoardRange(DeviceIndex, 0);

  BaseClock := 0;
  //BusyCt := 0;
  //MostBusyCt := 0;
  //MostQueueCt := 0;
  //FirstQueueCt := 0;

  //Register the callback to provide data.
  Status := HWave_RegisterCallback(DeviceIndex, DACallback);
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from HWave_RegisterCallback.');
    Exit;
  end;

  //Start the wave sequence. It will simply block until data is queued.
  Status := HWave_Start(DeviceIndex, Milliseconds);
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from HWave_Start.');
    Exit;
  end;

  LogMemo.Lines.Add('Started.');

  ////We queue in a fast-return idle handler. It's simple, but kinda dumb.
  //Application.OnIdle := IdleQueue;

  DeviceCombo.Enabled := False;
  MillisecondCombo.Enabled := False;

  StopButton.Enabled := True;
  GoButton.Enabled := False;

  StopButton.SetFocus;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
var
  Status: LongWord;

begin
  GoButton.Enabled := True;
  StopButton.Enabled := False;

  DeviceCombo.Enabled := True;
  MillisecondCombo.Enabled := True;

  Application.OnIdle := nil;

  //Stop the wave sequence. Note that this will discard any queued DAC data. If this
  //call returns an error, then the wave sequence isn't running anyways, so we
  //proceed even if it errors.
  Status := HWave_Stop(DeviceIndex);
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from HWave_Stop.');
  end;

  //Reset the DACs to mid-scale.
  DACMultiDirect(DeviceIndex, @MidScale[0], Length(MidScale));
  
  LogMemo.Lines.Add('Stopped.');

  //LogMemo.Lines.Add('Most Contiguous Busies: ' + IntToStr(MostBusyCt));
  //LogMemo.Lines.Add('First Contiguous Queues: ' + IntToStr(FirstQueueCt));
  //LogMemo.Lines.Add('Most Contiguous Queues Thereafter: ' + IntToStr(MostQueueCt));

  GoButton.SetFocus;
end;

{
procedure TForm1.IdleQueue(Sender: TObject; var Done: Boolean);
var
  Status: LongWord;
  iDAC, QueueCt: Integer;
  BaseTheta: Extended;
  DACData: array[$0..$F] of Word;

begin
  try

    QueueCt := 0;
    repeat
      //Generate the DAC data.
      BaseTheta := 2*Pi/19 * BaseClock;
      for iDAC := 0 to High(DACData) do
        DACData[iDAC] := Round($FFFF/2 * (1 + Sin(BaseTheta * (1 + iDAC))))
      ;
      //Queue the DAC data.
      Status := HWave_Queue(DeviceIndex, @DACData[0]);
      if Status <> ERROR_SUCCESS then Break;

      Inc(QueueCt);

      //If queue succeeded, then advance the clock for the next DAC data.
      BaseClock := (BaseClock + 1) mod 19;
    until False;

    //Update timing notes.
    Inc(BusyCt);
    if QueueCt <> 0 then begin
      if MostBusyCt < BusyCt then MostBusyCt := BusyCt;
      BusyCt := 0;
      if FirstQueueCt = 0 then
        FirstQueueCt := QueueCt
      else if MostQueueCt < QueueCt then
        MostQueueCt := QueueCt
      ;
    end;

    Done := False;
  except
    Application.OnIdle := nil;
    raise;
  end;
end;
}

procedure DACallback(pDACData: TPDACEntry; DACDataCount: PLongWord); cdecl;
var
  iDAC: Integer;
  BaseTheta: Extended;

begin
  //Generate the DAC data.
  BaseTheta := 2*Pi/19 * BaseClock;
  for iDAC := 0 to DACDataCount^-1 do begin
    pDACData^.Value := Round($FFFF/2 * (1 + Sin(BaseTheta * (1 + iDAC))));
    Inc(pDACData);
  end;
  BaseClock := (BaseClock + 1) mod 19;
end;

end.
